/*
c3hess - Console Chess written in C
Copyright (C) 2023 Kasyanov Nikolay Alexeyevich

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#ifndef algebraic_notation_h
#define algebraic_notation_h


#include "position.h"
#include "piece.h"
#include "move.h"


// A notated tile representation
typedef struct TileNotation {
    char file;
    unsigned int rank;
} TileNotation;


// Converts a tile notation to a position on the board
Position notation_tile_to_position(TileNotation notation);


// Converts notated tile into its string representation
char* notation_tile_to_string(TileNotation tile);


// Converts notated tile into its string representation on dest
void notation_tile_to_string_cpy(TileNotation tile, char* dest);


// Converts notated tile string into its corresponding structure
TileNotation notation_tile_from_string(const char* tile_string);


// Creates a new notation tile from given file and rank. 
TileNotation notation_tile_new(char file, unsigned int rank);


// Converts position to a notated position
TileNotation notation_tile_from_pos(Position pos);


// Returns a piece character. Upper case for white, lower case for black
char notation_piece_to_char(Piece piece);


// Returns 1 if character is one of the piece characters, 0 otherwise
int notation_is_piece_char(char character);

/*
Returns a piece deduced from character, returns NONE piece if couldn't deduce a piece type.
Big characters are for WHITE pieces, small for BLACK pieces.
*/
Piece notation_piece_from_char(char character);


// A notated move representation
typedef struct MoveNotation {
    Piece piece;
    TileNotation origin;
    TileNotation target;
    int move_type;
} MoveNotation;


// Converts notated move into its string representation
char* notation_move_to_string(MoveNotation move);


/*
Accepts a move string which consists of an origin square, piece name
(upper case for WHITE, lower case for BLACK) and a square to move to 
in algebraic notation.

Returns a notated move with origin, target and piece values set. Move type is to be
determined by the engine.

Returns NULL if technical errors were encountered, move_string does not contain enough information
*/
MoveNotation* notation_move_from_string(const char* move_string);


/*
Creates a new move notation from provided arguments
*/
MoveNotation notation_move_new(Piece piece, TileNotation origin, TileNotation target, int move_type);


// Converts move notation to a move
Move notation_to_move(MoveNotation notation);

#endif