/*
c3hess - Console Chess written in C
Copyright (C) 2023 Kasyanov Nikolay Alexeyevich

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#ifndef chess_h
#define chess_h


#include <stdio.h>
#include "board.h"


typedef enum GameState {
    READY,
    RUNNING,
    WHITEWON,
    BLACKWON
} GameState;


typedef struct Options {
    FILE* input;
    FILE* output;
    int use_ansi;
    int play_as_white;
} Options;


typedef struct Chess {
    Board* board;
    Color turn;
    Color plays_as;
    unsigned int moves_made;
    unsigned int timer_white_ms;
    unsigned int timer_black_ms;
    GameState state;
    Options options;
} Chess;


Chess* chess_new(Options options);


void chess_display(const Chess* chess);


Move* chess_take_move_input(Chess* chess);


void chess_toggle_turn(Chess* chess);


void chess_next_turn(Chess* chess);


void chess_delete(Chess** chess);


#endif