/*
c3hess - Console Chess written in C
Copyright (C) 2023 Kasyanov Nikolay Alexeyevich

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include "chess.h"


const char* VERSION = "0.1.0";

int main(int argc, const char** argv) {
    int use_ansi = 0;

    unsigned int i = 0;
    while (i < (unsigned int) argc) {
        if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0) {
            printf(
                "chess -[h|v|na]\n\n\
-h --help -> prints this help message\n\
-v --version -> prints version information\n\
-a --ansi -> use ANSI escape codes to output colors\n\
"
            );
            return 0;
        }

        if (strcmp(argv[i], "-v") == 0 || strcmp(argv[i], "--version") == 0) {
            printf(
                "c3hess v%s\n(C) 2023 Kasyanov Nikolay Alexeyevich (Unbewohnte)\n\n\
This program is free software: you can redistribute it and/or modify\n\
it under the terms of the GNU Affero General Public License as published by\n\
the Free Software Foundation, either version 3 of the License, or\n\
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU Affero General Public License for more details.\n\
\n\
You should have received a copy of the GNU Affero General Public License\n\
along with this program.  If not, see <https://www.gnu.org/licenses/>.\n", VERSION);
            return 0;
        }

        if (strcmp(argv[i], "-na") == 0 || strcmp(argv[i], "--no-ansi") == 0) {
            use_ansi = 1;
        }

        i++;
    } 

    Chess* game = chess_new((Options) {
        input: stdin,
        output: stdout,
        use_ansi: use_ansi
    });
    game->state = RUNNING;
    while (game->state == RUNNING) {
        chess_display(game);
        chess_next_turn(game);
    }
    chess_delete(&game);

    return 0;
}