/*
c3hess - Console Chess written in C
Copyright (C) 2023 Kasyanov Nikolay Alexeyevich

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#include "position.h"
#include "util.h"


// Returns a copy of a new position
Position position_new(unsigned int row, unsigned int column) {
    Position new_pos;
    new_pos.row = row;
    new_pos.column = column;

    return new_pos;
}


// Returns 1 if position (which) is diagonal to the other (to_which) position, 0 - otherwise
int position_is_diagonal_to(Position which, Position to_which) {
    if (_sub_abs(which.column, to_which.column) == _sub_abs(which.row, to_which.row)) {
        return 1;
    }

    return 0;
}


// Returns 1 if position (which) is in a straight (not diagonal) line to the other one (to_which), returns 0 if not
int position_is_straight_to(Position which, Position to_which) {
    if (which.row == to_which.row || which.column == to_which.column) {
        return 1;
    }
    
    return 0;
}


// Returns a diagonal distance between two positions
int position_diagonal_distance(Position which, Position to_which) {
    return _sub_abs(which.column, to_which.column);
}


int _max(int x, int y) {
    if (x > y) {
        return x;
    }

    return y;
}

// Returns a distance between the two tile positions
int position_distance(Position which, Position to_which) {
    // 0 0 - 1 1 gives 1
    // 0 0 - 2 0 gives 2
    // 0 0 - 0 3 gives 3
    return _max(_sub_abs(which.row, to_which.row), _sub_abs(which.column, to_which.column));
}


// Returns a position with the row incremented
Position position_up(Position pos) {
    return position_new(pos.row + 1, pos.column);
}

// Returns a position with the row decremented
Position position_down(Position pos) {
    if (pos.row == 0) {
        return pos;
    }

    return position_new(pos.row - 1, pos.column);
}

// Returns a position with the column decremented
Position position_left(Position pos) {
    if (pos.column == 0) {
        return pos;
    }

    return position_new(pos.row, pos.column - 1);
}

// Returns a position with the column incremented
Position position_right(Position pos) {
    return position_new(pos.row, pos.column + 1);
}


// Returns a position with the row incremented and column incremented
Position position_up_right(Position pos) {
    return position_new(pos.row + 1, pos.column + 1);
}


// Returns a position with the row incremented and column decremented
Position position_up_left(Position pos) {
    if (pos.column == 0) {
        return pos;
    }

    return position_new(pos.row + 1, pos.column - 1);
}

// Returns a position with the row decremented and column incremented
Position position_down_right(Position pos) {
    if (pos.row == 0) {
        return pos;
    }

    return position_new(pos.row - 1, pos.column + 1);
}


// Returns a position with the row decremented and column decremented
Position position_down_left(Position pos) {
    if (pos.row == 0 || pos.column == 0) {
        return pos;
    }

    return position_new(pos.row - 1, pos.column - 1);
}