/*
c3hess - Console Chess written in C
Copyright (C) 2023 Kasyanov Nikolay Alexeyevich

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#ifndef position_h
#define position_h

typedef struct Position {
    unsigned int row;
    unsigned int column;
} Position;


// Returns a copy of a new position
Position position_new(unsigned int row, unsigned int column);


// Returns 1 if position (which) is diagonal to the other (to_which) position, 0 - otherwise
int position_is_diagonal_to(Position which, Position to_which);


// Returns 1 if position (which) is in a straight (not diagonal) line to the other one (to_which), returns 0 if not
int position_is_straight_to(Position which, Position to_which);


// Returns a diagonal distance between two positions
int position_diagonal_distance(Position which, Position to_which);


// Returns a distance between the two tile positions
int position_distance(Position which, Position to_which);


// Returns a position with the row incremented
Position position_up(Position pos);


// Returns a position with the row decremented
Position position_down(Position pos);


// Returns a position with the column decremented
Position position_left(Position pos);


// Returns a position with the column incremented
Position position_right(Position pos);


// Returns a position with the row incremented and column incremented
Position position_up_right(Position pos);


// Returns a position with the row incremented and column decremented
Position position_up_left(Position pos);

// Returns a position with the row decremented and column incremented
Position position_down_right(Position pos);


// Returns a position with the row decremented and column decremented
Position position_down_left(Position pos);

#endif